@@ -34,17 +34,20 @@ sections are determined by the NcsSectionsFactory class. In the
 event the gaps are larger, this RawIO only provides the samples from the first section as belonging
 to one Segment.
 
-This RawIO presents only a single Block and Segment.
-:TODO: This should likely be changed to provide multiple segments and allow for
-multiple .Ncs files in a directory with differing section structures.
+If .Ncs files are loaded these determine the Segments of data to be loaded. Events and spiking data
+outside of Segments defined by .Ncs files will be ignored. To access all time point data in a
+single Segment load a session excluding .Ncs files.
+
+Continuous data streams are ordered by descending sampling rate.
+
+This RawIO presents only a single Block.
 
 Author: Julia Sprenger, Carlos Canova, Samuel Garcia, Peter N. Steinmetz.
 """
 
-
 from ..baserawio import (BaseRawIO, _signal_channel_dtype, _signal_stream_dtype,
-                _spike_channel_dtype, _event_channel_dtype)
-
+                         _spike_channel_dtype, _event_channel_dtype)
+from operator import itemgetter
 import numpy as np
 import os
 import pathlib
@@ -56,7 +59,7 @@ from neo.rawio.neuralynxrawio.nlxheader import NlxHeader
 
 
 class NeuralynxRawIO(BaseRawIO):
-    """"
+    """
     Class for reading datasets recorded by Neuralynx.
 
     This version works with rawmode of one-dir for a single directory of files or one-file for
@@ -89,8 +92,8 @@ class NeuralynxRawIO(BaseRawIO):
             name of directory containing all files for dataset. If provided, filename is
             ignored.
         filename: str
-            name of a single ncs, nse, nev, or ntt file to include in dataset. If used,
-            dirname must not be provided.
+            name of a single ncs, nse, nev, or ntt file to include in dataset. Will be ignored,
+            if dirname is provided.
         exclude_filename: str or list
             name of a single ncs, nse, nev or ntt file or list of such files. Expects plain
             filenames (without directory path).
@@ -117,9 +120,6 @@ class NeuralynxRawIO(BaseRawIO):
         else:
             return self.dirname
 
-    # from memory_profiler import profile
-    #
-    # @profile()
     def _parse_header(self):
 
         stream_channels = []
@@ -166,6 +166,8 @@ class NeuralynxRawIO(BaseRawIO):
                 if excl_file in filenames:
                     filenames.remove(excl_file)
 
+        stream_props = {}  # {(sampling_rate, n_samples, t_start): {stream_id: [filenames]}
+
         for filename in filenames:
             filename = os.path.join(dirname, filename)
 
@@ -189,17 +191,30 @@ class NeuralynxRawIO(BaseRawIO):
             for idx, chan_id in enumerate(chan_ids):
                 chan_name = chan_names[idx]
 
-                chan_uid = (chan_name, chan_id)
+                chan_uid = (chan_name, str(chan_id))
                 if ext == 'ncs':
+                    file_mmap = self._get_file_map(filename)
+                    n_packets = copy.copy(file_mmap.shape[0])
+                    if n_packets:
+                        t_start = copy.copy(file_mmap[0][0])
+                    else:  # empty file
+                        t_start = 0
+                    stream_prop = (info['sampling_rate'], n_packets, t_start)
+                    if stream_prop not in stream_props:
+                        stream_props[stream_prop] = {'stream_id': len(stream_props),
+                                                     'filenames': [filename]}
+                    else:
+                        stream_props[stream_prop]['filenames'].append(filename)
+                    stream_id = stream_props[stream_prop]['stream_id']
+
                     # a sampled signal channel
                     units = 'uV'
                     gain = info['bit_to_microVolt'][idx]
                     if info.get('input_inverted', False):
                         gain *= -1
                     offset = 0.
-                    stream_id = 0
                     signal_channels.append((chan_name, str(chan_id), info['sampling_rate'],
-                                         'int16', units, gain, offset, stream_id))
+                                            'int16', units, gain, offset, stream_id))
                     self.ncs_filenames[chan_uid] = filename
                     keys = [
                         'DspFilterDelay_µs',
@@ -281,12 +296,12 @@ class NeuralynxRawIO(BaseRawIO):
         spike_channels = np.array(spike_channels, dtype=_spike_channel_dtype)
         event_channels = np.array(event_channels, dtype=_event_channel_dtype)
 
-        # require all sampled signals, ncs files, to have the same sampling rate
         if signal_channels.size > 0:
-            sampling_rate = np.unique(signal_channels['sampling_rate'])
-            assert sampling_rate.size == 1
-            self._sigs_sampling_rate = sampling_rate[0]
-            signal_streams = [('signals', '0')]
+            # ordering streams according from high to low sampling rates
+            stream_props = {k: stream_props[k] for k in sorted(stream_props, reverse=True)}
+            names = [f'Stream (rate,#packet,t0): {sp}' for sp in stream_props]
+            ids = [stream_prop['stream_id'] for stream_prop in stream_props.values()]
+            signal_streams = list(zip(names, ids))
         else:
             signal_streams = []
         signal_streams = np.array(signal_streams, dtype=_signal_stream_dtype)
@@ -296,17 +311,84 @@ class NeuralynxRawIO(BaseRawIO):
         self._timestamp_limits = None
         self._nb_segment = 1
 
-        # Read ncs files for gap detection and nb_segment computation.
-        self._sigs_memmaps, ncsSegTimestampLimits = self.scan_ncs_files(self.ncs_filenames)
-        if ncsSegTimestampLimits:
-            self._ncs_seg_timestamp_limits = ncsSegTimestampLimits  # save copy
-            self._nb_segment = ncsSegTimestampLimits.nb_segment
-            self._sigs_length = ncsSegTimestampLimits.length.copy()
-            self._timestamp_limits = ncsSegTimestampLimits.timestamp_limits.copy()
-            self._sigs_t_start = ncsSegTimestampLimits.t_start.copy()
-            self._sigs_t_stop = ncsSegTimestampLimits.t_stop.copy()
+        stream_infos = {}
 
-        # Determine timestamp limits in nev, nse file by scanning them.
+        # Read ncs files of each stream for gap detection and nb_segment computation.
+        for stream_id in np.unique(signal_channels['stream_id']):
+            stream_channels = signal_channels[signal_channels['stream_id'] == stream_id]
+            stream_chan_uids = zip(stream_channels['name'], stream_channels['id'])
+            stream_filenames = [self.ncs_filenames[chuid] for chuid in stream_chan_uids]
+            _sigs_memmaps, ncsSegTimestampLimits, section_structure = self.scan_stream_ncs_files(
+                stream_filenames)
+
+            stream_infos[stream_id] = {'segment_sig_memmaps': _sigs_memmaps,
+                                       'ncs_segment_infos': ncsSegTimestampLimits,
+                                       'section_structure': section_structure}
+
+        # check if section structure across streams is compatible and merge infos
+        ref_sec_structure = None
+        for stream_id, stream_info in stream_infos.items():
+            ref_stream_id = list(stream_infos.keys())[0]
+            ref_sec_structure = stream_infos[ref_stream_id]['section_structure']
+
+            sec_structure = stream_info['section_structure']
+
+            # check if section structure of streams are compatible
+            # using tolerance of one data packet (512 samples)
+            tolerance = 512 / min(ref_sec_structure.sampFreqUsed,
+                                  sec_structure.sampFreqUsed) * 1e6
+            if not ref_sec_structure.is_equivalent(sec_structure, abs_tol=tolerance):
+                ref_chan_ids = signal_channels[signal_channels['stream_id'] == ref_stream_id][
+                    'name']
+                chan_ids = signal_channels[signal_channels['stream_id'] == stream_id]['name']
+
+                raise ValueError('Incompatible section structures across streams: '
+                                 f'Stream id {ref_stream_id}:{ref_chan_ids} and '
+                                 f'{stream_id}:{chan_ids}.')
+
+        if ref_sec_structure is not None:
+            self._nb_segment = len(ref_sec_structure.sects)
+        else:
+            # Use only a single segment if no ncs data is present
+            self._nb_segment = 1
+
+        def min_max_tuple(tuple1, tuple2):
+            """Merge tuple by selecting min for first and max for 2nd entry"""
+            mins, maxs = zip(tuple1, tuple2)
+            result = (min(m for m in mins if m is not None), max(m for m in maxs if m is not None))
+            return result
+
+        # merge stream mmemmaps since streams are compatible
+        self._sigs_memmaps = [{} for seg_idx in range(self._nb_segment)]
+        # time limits of integer timestamps in ncs files
+        self._timestamp_limits = [(None, None) for seg_idx in range(self._nb_segment)]
+        # time limits physical times in ncs files
+        self._signal_limits = [(None, None) for seg_idx in range(self._nb_segment)]
+        for stream_id, stream_info in stream_infos.items():
+            stream_mmaps = stream_info['segment_sig_memmaps']
+            for seg_idx, signal_dict in enumerate(stream_mmaps):
+                self._sigs_memmaps[seg_idx].update(signal_dict)
+
+            ncs_segment_info = stream_info['ncs_segment_infos']
+            for seg_idx, (t_start, t_stop) in enumerate(ncs_segment_info.timestamp_limits):
+                self._timestamp_limits[seg_idx] = min_max_tuple(self._timestamp_limits[seg_idx],
+                                                                (t_start, t_stop))
+
+            for seg_idx in range(ncs_segment_info.nb_segment):
+                t_start = ncs_segment_info.t_start[seg_idx]
+                t_stop = ncs_segment_info.t_stop[seg_idx]
+                self._signal_limits[seg_idx] = min_max_tuple(self._signal_limits[seg_idx],
+                                                             (t_start, t_stop))
+
+        # precompute signal lengths within segments
+        self._sigs_length = []
+        if self._sigs_memmaps:
+            for seg_idx, sig_container in enumerate(self._sigs_memmaps):
+                self._sigs_length.append({})
+                for chan_uid, sig_infos in sig_container.items():
+                    self._sigs_length[seg_idx][chan_uid] = int(sig_infos['nb_valid'].sum())
+
+        # Determine timestamp limits in nev, nse, ntt files by scanning them.
         ts0, ts1 = None, None
         for _data_memmap in (self._spike_memmap, self._nev_memmap):
             for _, data in _data_memmap.items():
@@ -319,28 +401,36 @@ class NeuralynxRawIO(BaseRawIO):
                 ts0 = min(ts0, ts[0])
                 ts1 = max(ts1, ts[-1])
 
+        # rescaling for comparison with signal times
+        if ts0 is not None:
+            timestamps_start, timestamps_stop = ts0 / 1e6, ts1 / 1e6
+
         # decide on segment and global start and stop times based on files available
         if self._timestamp_limits is None:
-            # case  NO ncs but HAVE nev or nse
+            # case  NO ncs but HAVE nev or nse -> single segment covering all spikes & events
             self._timestamp_limits = [(ts0, ts1)]
-            self._seg_t_starts = [ts0 / 1e6]
-            self._seg_t_stops = [ts1 / 1e6]
-            self.global_t_start = ts0 / 1e6
-            self.global_t_stop = ts1 / 1e6
+            self._seg_t_starts = [timestamps_start]
+            self._seg_t_stops = [timestamps_stop]
+            self.global_t_start = timestamps_start
+            self.global_t_stop = timestamps_stop
         elif ts0 is not None:
-            # case  HAVE ncs AND HAVE nev or nse
-            self.global_t_start = min(ts0 / 1e6, self._sigs_t_start[0])
-            self.global_t_stop = max(ts1 / 1e6, self._sigs_t_stop[-1])
-            self._seg_t_starts = list(self._sigs_t_start)
+            # case  HAVE ncs AND HAVE nev or nse -> multi segments based on ncs segmentation
+            # ignoring nev/nse/ntt time limits, loading only data within ncs segments
+            global_events_limits = (timestamps_start, timestamps_stop)
+            global_signal_limits = (self._signal_limits[0][0], self._signal_limits[-1][-1])
+            self.global_t_start, self.global_t_stop = min_max_tuple(global_events_limits,
+                                                                    global_signal_limits)
+            self._seg_t_starts = [limits[0] for limits in self._signal_limits]
+            self._seg_t_stops = [limits[1] for limits in self._signal_limits]
             self._seg_t_starts[0] = self.global_t_start
-            self._seg_t_stops = list(self._sigs_t_stop)
             self._seg_t_stops[-1] = self.global_t_stop
+
         else:
-            # case HAVE ncs but  NO nev or nse
-            self._seg_t_starts = self._sigs_t_start
-            self._seg_t_stops = self._sigs_t_stop
-            self.global_t_start = self._sigs_t_start[0]
-            self.global_t_stop = self._sigs_t_stop[-1]
+            # case HAVE ncs but  NO nev or nse ->
+            self._seg_t_starts = [limits[0] for limits in self._signal_limits]
+            self._seg_t_stops = [limits[1] for limits in self._signal_limits]
+            self.global_t_start = self._signal_limits[0][0]
+            self.global_t_stop = self._signal_limits[-1][-1]
 
         if self.keep_original_times:
             self.global_t_stop = self.global_t_stop - self.global_t_start
@@ -362,19 +452,20 @@ class NeuralynxRawIO(BaseRawIO):
         for seg_index in range(self._nb_segment):
             seg_annotations = bl_annotations['segments'][seg_index]
 
-            for c in range(signal_streams.size):
+            for stream_id in range(signal_streams.size):
                 # one or no signal stream
-                sig_ann = seg_annotations['signals'][c]
+                stream_ann = seg_annotations['signals'][stream_id]
                 # handle array annotations
                 for key in signal_annotations[0].keys():
                     values = []
-                    for c in range(signal_channels.size):
-                        value = signal_annotations[0][key]
+                    # only collect values from channels belonging to current stream
+                    for d in np.where(signal_channels['stream_id'] == f'{stream_id}')[0]:
+                        value = signal_annotations[d][key]
                         values.append(value)
                     values = np.array(values)
                     if values.ndim == 1:
                         # 'InputRange': is 2D and make bugs
-                        sig_ann['__array_annotations__'][key] = values
+                        stream_ann['__array_annotations__'][key] = values
 
             for c in range(spike_channels.size):
                 unit_ann = seg_annotations['spikes'][c]
@@ -403,7 +494,7 @@ class NeuralynxRawIO(BaseRawIO):
 
         if suffix == 'ncs':
             return np.memmap(filename, dtype=self._ncs_dtype, mode='r',
-                      offset=NlxHeader.HEADER_SIZE)
+                             offset=NlxHeader.HEADER_SIZE)
 
         elif suffix in ['nse', 'ntt']:
             info = NlxHeader(filename)
@@ -432,17 +523,36 @@ class NeuralynxRawIO(BaseRawIO):
         return self._seg_t_stops[seg_index] - self.global_t_start
 
     def _get_signal_size(self, block_index, seg_index, stream_index):
-        return self._sigs_length[seg_index]
+        stream_id = self.header['signal_streams'][stream_index]['id']
+        stream_mask = self.header['signal_channels']['stream_id'] == stream_id
+        signals = self.header['signal_channels'][stream_mask]
+
+        if len(signals):
+            sig = signals[0]
+            return self._sigs_length[seg_index][(sig['name'], sig['id'])]
+        else:
+            raise ValueError(f'No signals present for block {block_index}, segment {seg_index},'
+                             f' stream {stream_index}')
 
     def _get_signal_t_start(self, block_index, seg_index, stream_index):
-        return self._sigs_t_start[seg_index] - self.global_t_start
+
+        stream_id = self.header['signal_streams'][stream_index]['id']
+        stream_mask = self.header['signal_channels']['stream_id'] == stream_id
+
+        # use first channel of stream as all channels in stream have a common t_start
+        channel = self.header['signal_channels'][stream_mask][0]
+
+        data = self._sigs_memmaps[seg_index][(channel['name'], channel['id'])]
+        absolute_t_start = data['timestamp'][0]
+
+        return absolute_t_start / 1e6 - self.global_t_start
 
     def _get_analogsignal_chunk(self, block_index, seg_index, i_start, i_stop,
                                 stream_index, channel_indexes):
         """
         Retrieve chunk of analog signal, a chunk being a set of contiguous samples.
 
-        PARAMETERS
+        Parameters
         ----------
         block_index:
             index of block in dataset, ignored as only 1 block in this implementation
@@ -455,14 +565,15 @@ class NeuralynxRawIO(BaseRawIO):
         channel_indexes:
             list of channel indices to return data for
 
-        RETURNS
+        Returns
         -------
             array of samples, with each requested channel in a column
         """
         if i_start is None:
             i_start = 0
         if i_stop is None:
-            i_stop = self._sigs_length[seg_index]
+            i_stop = self.get_signal_size(block_index=block_index, seg_index=seg_index,
+                                          stream_index=stream_index)
 
         block_start = i_start // NcsSection._RECORD_SIZE
         block_stop = i_stop // NcsSection._RECORD_SIZE + 1
@@ -472,8 +583,11 @@ class NeuralynxRawIO(BaseRawIO):
         if channel_indexes is None:
             channel_indexes = slice(None)
 
-        channel_ids = self.header['signal_channels'][channel_indexes]['id'].astype(int)
-        channel_names = self.header['signal_channels'][channel_indexes]['name']
+        stream_id = self.header['signal_streams'][stream_index]['id']
+        stream_mask = self.header['signal_channels']['stream_id'] == stream_id
+
+        channel_ids = self.header['signal_channels'][stream_mask][channel_indexes]['id']
+        channel_names = self.header['signal_channels'][stream_mask][channel_indexes]['name']
 
         # create buffer for samples
         sigs_chunk = np.zeros((i_stop - i_start, len(channel_ids)), dtype='int16')
@@ -490,7 +604,13 @@ class NeuralynxRawIO(BaseRawIO):
         data = self._spike_memmap[chan_uid]
         ts = data['timestamp']
 
-        ts0, ts1 = self._timestamp_limits[seg_index]
+        ts0 = self.segment_t_start(block_index, seg_index)
+        ts1 = self.segment_t_stop(block_index, seg_index)
+
+        # rescale to integer sampling of time
+        ts0 = int((ts0 + self.global_t_start) * 1e6)
+        ts1 = int((ts1 + self.global_t_start) * 1e6)
+
 
         # only count spikes inside the timestamp limits, inclusive, and for the specified unit
         keep = (ts >= ts0) & (ts <= ts1) & (unit_id == data['unit_id'])
@@ -498,15 +618,22 @@ class NeuralynxRawIO(BaseRawIO):
         return nb_spike
 
     def _get_spike_timestamps(self, block_index, seg_index, unit_index, t_start, t_stop):
+        """
+        Extract timestamps within a Segment defined by ncs timestamps
+        """
         chan_uid, unit_id = self.internal_unit_ids[unit_index]
         data = self._spike_memmap[chan_uid]
         ts = data['timestamp']
 
-        ts0, ts1 = self._timestamp_limits[seg_index]
-        if t_start is not None:
-            ts0 = int((t_start + self.global_t_start) * 1e6)
-        if t_start is not None:
-            ts1 = int((t_stop + self.global_t_start) * 1e6)
+        ts0, ts1 = t_start, t_stop
+        if ts0 is None:
+            ts0 = self.segment_t_start(block_index, seg_index)
+        if ts1 is None:
+            ts1 = self.segment_t_stop(block_index, seg_index)
+
+        # rescale to integer sampling of time
+        ts0 = int((ts0 + self.global_t_start) * 1e6)
+        ts1 = int((ts1 + self.global_t_start) * 1e6)
 
         keep = (ts >= ts0) & (ts <= ts1) & (unit_id == data['unit_id'])
         timestamps = ts[keep]
@@ -518,17 +645,20 @@ class NeuralynxRawIO(BaseRawIO):
         spike_times -= self.global_t_start
         return spike_times
 
-    def _get_spike_raw_waveforms(self, block_index, seg_index, unit_index,
-                                 t_start, t_stop):
+    def _get_spike_raw_waveforms(self, block_index, seg_index, unit_index, t_start, t_stop):
         chan_uid, unit_id = self.internal_unit_ids[unit_index]
         data = self._spike_memmap[chan_uid]
         ts = data['timestamp']
 
-        ts0, ts1 = self._timestamp_limits[seg_index]
-        if t_start is not None:
-            ts0 = int((t_start + self.global_t_start) * 1e6)
-        if t_start is not None:
-            ts1 = int((t_stop + self.global_t_start) * 1e6)
+        ts0, ts1 = t_start, t_stop
+        if ts0 is None:
+            ts0 = self.segment_t_start(block_index, seg_index)
+        if ts1 is None:
+            ts1 = self.segment_t_stop(block_index, seg_index)
+
+        # rescale to integer sampling of time
+        ts0 = int((ts0 + self.global_t_start) * 1e6)
+        ts1 = int((ts1 + self.global_t_start) * 1e6)
 
         keep = (ts >= ts0) & (ts <= ts1) & (unit_id == data['unit_id'])
 
@@ -546,7 +676,14 @@ class NeuralynxRawIO(BaseRawIO):
         event_id, ttl_input = self.internal_event_ids[event_channel_index]
         chan_id = self.header['event_channels'][event_channel_index]['id']
         data = self._nev_memmap[chan_id]
-        ts0, ts1 = self._timestamp_limits[seg_index]
+
+        ts0 = self.segment_t_start(block_index, seg_index)
+        ts1 = self.segment_t_stop(block_index, seg_index)
+
+        # rescale to integer sampling of time
+        ts0 = int((ts0 + self.global_t_start) * 1e6)
+        ts1 = int((ts1 + self.global_t_start) * 1e6)
+
         ts = data['timestamp']
         keep = (ts >= ts0) & (ts <= ts1) & (data['event_id'] == event_id) & \
                (data['ttl_input'] == ttl_input)
@@ -557,12 +694,16 @@ class NeuralynxRawIO(BaseRawIO):
         event_id, ttl_input = self.internal_event_ids[event_channel_index]
         chan_id = self.header['event_channels'][event_channel_index]['id']
         data = self._nev_memmap[chan_id]
-        ts0, ts1 = self._timestamp_limits[seg_index]
 
-        if t_start is not None:
-            ts0 = int((t_start + self.global_t_start) * 1e6)
-        if t_start is not None:
-            ts1 = int((t_stop + self.global_t_start) * 1e6)
+        ts0, ts1 = t_start, t_stop
+        if ts0 is None:
+            ts0 = self.segment_t_start(block_index, seg_index)
+        if ts1 is None:
+            ts1 = self.segment_t_stop(block_index, seg_index)
+
+        # rescale to integer sampling of time
+        ts0 = int((ts0 + self.global_t_start) * 1e6)
+        ts1 = int((ts1 + self.global_t_start) * 1e6)
 
         ts = data['timestamp']
         keep = (ts >= ts0) & (ts <= ts1) & (data['event_id'] == event_id) & \
@@ -580,45 +721,52 @@ class NeuralynxRawIO(BaseRawIO):
         event_times -= self.global_t_start
         return event_times
 
-    def scan_ncs_files(self, ncs_filenames):
+    def scan_stream_ncs_files(self, ncs_filenames):
         """
         Given a list of ncs files, read their basic structure.
+        Ncs files have to have common sampling_rate, number of packets and t_start
+        (be part of a single stream)
 
-        PARAMETERS:
-        ------
-        ncs_filenames - list of ncs filenames to scan.
+        Parameters
+        ----------
+        ncs_filenames: list
+            List of ncs filenames to scan.
 
-        RETURNS:
-        ------
+        Returns
+        -------
         memmaps
             [ {} for seg_index in range(self._nb_segment) ][chan_uid]
         seg_time_limits
             SegmentTimeLimits for sections in scanned Ncs files
+        section_structure
+            Section structure common to the ncs files
 
         Files will be scanned to determine the sections of records. If file is a single
         section of records, this scan is brief, otherwise it will check each record which may
         take some time.
         """
 
-        # :TODO: Needs to account for gaps and start and end times potentially
-        #    being different in different groups of channels. These groups typically
-        #    correspond to the channels collected by a single ADC card.
         if len(ncs_filenames) == 0:
-            return None, None
+            return None, None, None
 
         # Build dictionary of chan_uid to associated NcsSections, memmap and NlxHeaders. Only
         # construct new NcsSections when it is different from that for the preceding file.
         chanSectMap = dict()
-        for chan_uid, ncs_filename in self.ncs_filenames.items():
+        sig_length = []
+        for ncs_filename in ncs_filenames:
 
             data = self._get_file_map(ncs_filename)
             nlxHeader = NlxHeader(ncs_filename)
 
-            if not chanSectMap or (chanSectMap and
-                    not NcsSectionsFactory._verifySectionsStructure(data,
-                    lastNcsSections)):
-                lastNcsSections = NcsSectionsFactory.build_for_ncs_file(data, nlxHeader)
-            chanSectMap[chan_uid] = [lastNcsSections, nlxHeader, ncs_filename]
+            verify_sec_struct = NcsSectionsFactory._verifySectionsStructure
+            if not chanSectMap or (not verify_sec_struct(data, chan_ncs_sections)):
+                chan_ncs_sections = NcsSectionsFactory.build_for_ncs_file(data, nlxHeader)
+
+            # register file section structure for all contained channels
+            for chan_uid in zip(nlxHeader['channel_names'],
+                                np.asarray(nlxHeader['channel_ids'], dtype=str)):
+                chanSectMap[chan_uid] = [chan_ncs_sections, nlxHeader, ncs_filename]
+
             del data
 
         # Construct an inverse dictionary from NcsSections to list of associated chan_uids
@@ -630,16 +778,16 @@ class NeuralynxRawIO(BaseRawIO):
         # be one entry. Otherwise this is presently unsupported.
         if len(revSectMap) > 1:
             raise IOError(f'ncs files have {len(revSectMap)} different sections '
-                          f'structures. Unsupported configuration.')
+                          f'structures. Unsupported configuration to be handled with in a single '
+                          f'stream.')
 
-        seg_time_limits = SegmentTimeLimits(nb_segment=len(lastNcsSections.sects),
+        seg_time_limits = SegmentTimeLimits(nb_segment=len(chan_ncs_sections.sects),
                                             t_start=[], t_stop=[], length=[],
                                             timestamp_limits=[])
         memmaps = [{} for seg_index in range(seg_time_limits.nb_segment)]
 
         # create segment with subdata block/t_start/t_stop/length for each channel
-        for i, fileEntry in enumerate(self.ncs_filenames.items()):
-            chan_uid = fileEntry[0]
+        for i, chan_uid in enumerate(chanSectMap.keys()):
             data = self._get_file_map(chanSectMap[chan_uid][2])
 
             # create a memmap for each record section of the current file
@@ -668,14 +816,15 @@ class NeuralynxRawIO(BaseRawIO):
                     length = (subdata.size - 1) * NcsSection._RECORD_SIZE + numSampsLastSect
                     seg_time_limits.length.append(length)
 
-        return memmaps, seg_time_limits
+        stream_section_structure = list(revSectMap.keys())[0]
+
+        return memmaps, seg_time_limits, stream_section_structure
 
 
 # time limits for set of segments
 SegmentTimeLimits = namedtuple("SegmentTimeLimits", ['nb_segment', 't_start', 't_stop', 'length',
                                                      'timestamp_limits'])
 
-
 nev_dtype = [
     ('reserved', '<i2'),
     ('system_id', '<i2'),
