@@ -6,7 +6,7 @@ In this format channels are interleaved in one file.
 See
 https://open-ephys.github.io/gui-docs/User-Manual/Recording-data/Binary-format.html
 
-Author: Julia Sprenger and Samuel Garcia
+Author: Julia Sprenger, Samuel Garcia, and Alessio Buccino
 """
 
 
@@ -26,6 +26,23 @@ class OpenEphysBinaryRawIO(BaseRawIO):
     """
     Handle several Blocks and several Segments.
 
+    Parameters
+    ----------
+    dirname : str
+        Path to Open Ephys directory
+    load_sync_channel : bool
+        If False (default) and a SYNC channel is present (e.g. Neuropixels), this is not loaded.
+        If True, the SYNC channel is loaded and can be accessed in the analog signals.
+    experiment_names : str or list or None
+        If multiple experiments are available, this argument allows users to select one
+        or more experiments. If None, all experiements are loaded as blocks.
+        E.g. `experiment_names="experiment2"`, `experiment_names=["experiment1", "experiment2"]`
+
+    Note
+    ----
+    For multi-experiment datasets, the streams need to be consistent across experiments.
+    If this is not the case, you can select a subset of experiments with the `experiment_names`
+    argument.
 
     # Correspondencies
     Neo          OpenEphys
@@ -40,20 +57,30 @@ class OpenEphysBinaryRawIO(BaseRawIO):
     extensions = []
     rawmode = 'one-dir'
 
-    def __init__(self, dirname=''):
+    def __init__(self, dirname='', load_sync_channel=False, experiment_names=None):
         BaseRawIO.__init__(self)
         self.dirname = dirname
+        if experiment_names is not None:
+            if isinstance(experiment_names, str):
+                experiment_names = [experiment_names]
+        self.experiment_names = experiment_names
+        self.load_sync_channel = load_sync_channel
+        self.folder_structure = None
 
     def _source_name(self):
         return self.dirname
 
     def _parse_header(self):
-        all_streams, nb_block, nb_segment_per_block = explore_folder(self.dirname)
+        folder_structure, all_streams, nb_block, nb_segment_per_block, possible_experiments = \
+            explore_folder(self.dirname, self.experiment_names)
+        check_folder_consistency(folder_structure, possible_experiments)
+        self.folder_structure = folder_structure
 
+        # all streams are consistent across blocks and segments
         sig_stream_names = sorted(list(all_streams[0][0]['continuous'].keys()))
         event_stream_names = sorted(list(all_streams[0][0]['events'].keys()))
 
-        # first loop to reasign stream by "stream_index" instead of "stream_name"
+        # first loop to reassign stream by "stream_index" instead of "stream_name"
         self._sig_streams = {}
         self._evt_streams = {}
         for block_index in range(nb_block):
@@ -81,8 +108,15 @@ class OpenEphysBinaryRawIO(BaseRawIO):
             new_channels = []
             for chan_info in d['channels']:
                 chan_id = chan_info['channel_name']
+                if "SYNC" in chan_id and not self.load_sync_channel:
+                    continue
+                if chan_info["units"] == "":
+                    # in some cases for some OE version the unit is "", but the gain is to "uV"
+                    units = "uV"
+                else:
+                    units = chan_info["units"]
                 new_channels.append((chan_info['channel_name'],
-                    chan_id, float(d['sample_rate']), d['dtype'], chan_info['units'],
+                    chan_id, float(d['sample_rate']), d['dtype'], units,
                     chan_info['bit_volts'], 0., stream_id))
             signal_channels.extend(new_channels)
         signal_channels = np.array(signal_channels, dtype=_signal_channel_dtype)
@@ -99,9 +133,25 @@ class OpenEphysBinaryRawIO(BaseRawIO):
                 for stream_index, d in self._sig_streams[block_index][seg_index].items():
                     num_channels = len(d['channels'])
                     memmap_sigs = np.memmap(d['raw_filename'], d['dtype'],
-                                 order='C', mode='r').reshape(-1, num_channels)
+                                            order='C', mode='r').reshape(-1, num_channels)
+                    channel_names = [ch["channel_name"] for ch in d["channels"]]
+                    # if there is a sync channel and it should not be loaded,
+                    # find the right channel index and slice the memmap
+                    if any(["SYNC" in ch for ch in channel_names]) and \
+                        not self.load_sync_channel:
+                        sync_channel_name = [ch for ch in channel_names if "SYNC" in ch][0]
+                        sync_channel_index = channel_names.index(sync_channel_name)
+
+                        # only sync channel in last position is supported to keep memmap
+                        if sync_channel_index == num_channels - 1:
+                            memmap_sigs = memmap_sigs[:, :-1]
+                        else:
+                            raise NotImplementedError("SYNC channel removal is only supported "
+                                                      "when the sync channel is in the last "
+                                                      "position")
                     d['memmap'] = memmap_sigs
 
+
         # events zone
         # channel map: one channel one stream
         event_channels = []
@@ -136,6 +186,9 @@ class OpenEphysBinaryRawIO(BaseRawIO):
             elif 'channels' in d:
                 # ttl case use channels
                 d['labels'] = d['channels'].astype('U')
+            elif 'states' in d:
+                # ttl case use states
+                d['labels'] = d['states'].astype('U')
             else:
                 raise ValueError(f'There is no possible labels for this event: {stream_name}')
 
@@ -295,77 +348,118 @@ class OpenEphysBinaryRawIO(BaseRawIO):
         pass
 
 
-_possible_event_stream_names = ('timestamps', 'channels', 'text',
-        'full_word', 'channel_states', 'data_array', 'metadata')
+_possible_event_stream_names = ('timestamps', 'channels', 'text', 'states',
+                                'full_word', 'channel_states', 'data_array', 'metadata')
 
 
-def explore_folder(dirname):
+def explore_folder(dirname, experiment_names=None):
     """
-    Exploring the OpenEphys folder structure and structure.oebin
+    Exploring the OpenEphys folder structure, by looping through the 
+    folder to find recordings.
 
-    Returns nested dictionary structure:
-    [block_index][seg_index][stream_type][stream_information]
-    where
-    - node_name is the open ephys node id
-    - block_index is the neo Block index
-    - segment_index is the neo Segment index
-    - stream_type can be 'continuous'/'events'/'spikes'
-    - stream_information is a dictionionary containing e.g. the sampling rate
-
-    Parmeters
-    ---------
+    Parameters
+    ----------
     dirname (str): Root folder of the dataset
 
     Returns
     -------
-    nested dictionaries containing structure and stream information
+    folder_structure: dict
+        The folder_structure is dictionary that describes the Open Ephys folder.
+        Dictionary structure:
+        [node_name]["experiments"][exp_id]["recordings"][rec_id][stream_type][stream_information]
+    all_streams: dict
+        From the folder_structure, the another dictionary is reorganized with NEO-like
+        indexing: block_index (experiments) and seg_index (recordings):
+        Dictionary structure:
+        [block_index][seg_index][stream_type][stream_information]
+        where
+        - node_name is the open ephys node id
+        - block_index is the neo Block index
+        - segment_index is the neo Segment index
+        - stream_type can be 'continuous'/'events'/'spikes'
+        - stream_information is a dictionary containing e.g. the sampling rate
+    nb_block : int
+        Number of blocks (experiments) loaded
+    nb_segment_per_block : dict
+        Dictionary with number of segment per block.
+        Keys are block indices, values are number of segments
+    possible_experiment_names : list
+        List of all available experiments in the Open Ephys folder
     """
-    nb_block = 0
-    nb_segment_per_block = []
-    # nested dictionary: block_index > seg_index > data_type > stream_name
-    all_streams = {}
+    # folder with nodes, experiments, setting files, recordings, and streams
+    folder_structure = {}
+    possible_experiment_names = []
+
     for root, dirs, files in os.walk(dirname):
         for file in files:
             if not file == 'structure.oebin':
                 continue
             root = Path(root)
 
-            node_name = root.parents[1].stem
+            node_folder = root.parents[1]
+            node_name = node_folder.stem
             if not node_name.startswith('Record'):
                 # before version 5.x.x there was not multi Node recording
                 # so no node_name
                 node_name = ''
 
-            block_index = int(root.parents[0].stem.lower().replace('experiment', '')) - 1
-            if block_index not in all_streams:
-                all_streams[block_index] = {}
-                if block_index >= nb_block:
-                    nb_block = block_index + 1
-                    nb_segment_per_block.append(0)
+            if node_name not in folder_structure:
+                folder_structure[node_name] = {}
+                folder_structure[node_name]['experiments'] = {}
 
-            seg_index = int(root.stem.replace('recording', '')) - 1
-            if seg_index not in all_streams[block_index]:
-                all_streams[block_index][seg_index] = {
-                    'continuous': {},
-                    'events': {},
-                    'spikes': {},
-                }
-                if seg_index >= nb_segment_per_block[block_index]:
-                    nb_segment_per_block[block_index] = seg_index + 1
+            # here we skip if experiment_names is not None
+            experiment_folder = root.parents[0]
+            experiment_name = experiment_folder.stem
+            experiment_id = int(experiment_name.replace('experiment', ''))
+            if experiment_name not in possible_experiment_names:
+                possible_experiment_names.append(experiment_name)
+            if experiment_names is not None and experiment_name not in experiment_names:
+                continue
+            if experiment_id not in folder_structure[node_name]['experiments']:
+                experiment = {}
+                experiment['name'] = experiment_name
+                if experiment_name == 'experiment1':
+                    settings_file = node_folder / "settings.xml"
+                else:
+                    settings_file = node_folder / f"settings_{experiment_id}.xml"
+                experiment['settings_file'] = settings_file
+                experiment['recordings'] = {}
+                folder_structure[node_name]['experiments'][experiment_id] = experiment
+
+            recording_folder = root
+            recording_name = root.stem
+            recording_id = int(recording_name.replace('recording', ''))
+            # add recording
+            recording = {}
+            recording['name'] = recording_name
+            recording['streams'] = {}
 
             # metadata
-            with open(root / 'structure.oebin', encoding='utf8', mode='r') as f:
-                structure = json.load(f)
+            with open(recording_folder / 'structure.oebin', encoding='utf8', mode='r') as f:
+                rec_structure = json.load(f)
 
-            if (root / 'continuous').exists() and len(structure['continuous']) > 0:
-                for d in structure['continuous']:
+            if (recording_folder / 'continuous').exists() and len(rec_structure['continuous']) > 0:
+                recording['streams']['continuous'] = {}
+                for d in rec_structure['continuous']:
                     # when multi Record Node the stream name also contains
                     # the node name to make it unique
-                    stream_name = node_name + '#' + d['folder_name']
+                    oe_stream_name = Path(d["folder_name"]).name # remove trailing slash
+                    if len(node_name) > 0:
+                        stream_name = node_name + '#' + oe_stream_name
+                    else:
+                        stream_name = oe_stream_name
+                    raw_filename = recording_folder / 'continuous' / d['folder_name'] / 'continuous.dat'
 
-                    raw_filename = root / 'continuous' / d['folder_name'] / 'continuous.dat'
-
-                    timestamp_file = root / 'continuous' / d['folder_name'] / 'timestamps.npy'
+                    # Updates for OpenEphys v0.6:
+                    # In new vesion (>=0.6) timestamps.npy is now called sample_numbers.npy
+                    # see https://open-ephys.github.io/gui-docs/User-Manual/Recording-data/Binary-format.html#continuous
+                    sample_numbers = recording_folder / 'continuous' / d['folder_name'] / \
+                        'sample_numbers.npy'
+                    if sample_numbers.is_file():
+                        timestamp_file = sample_numbers
+                    else:
+                        timestamp_file = recording_folder / 'continuous' / d['folder_name'] / \
+                            'timestamps.npy'
                     timestamps = np.load(str(timestamp_file), mmap_mode='r')
                     timestamp0 = timestamps[0]
                     t_start = timestamp0 / d['sample_rate']
@@ -377,20 +471,100 @@ def explore_folder(dirname):
                     signal_stream['timestamp0'] = timestamp0
                     signal_stream['t_start'] = t_start
 
-                    all_streams[block_index][seg_index]['continuous'][stream_name] = signal_stream
+                    recording['streams']['continuous'][stream_name] = signal_stream
 
-            if (root / 'events').exists() and len(structure['events']) > 0:
-                for d in structure['events']:
-                    stream_name = node_name + '#' + d['folder_name']
+            if (root / 'events').exists() and len(rec_structure['events']) > 0:
+                recording['streams']['events'] = {}
+                for d in rec_structure['events']:
+                    oe_stream_name = Path(d["folder_name"]).name # remove trailing slash
+                    stream_name = node_name + '#' + oe_stream_name
 
                     event_stream = d.copy()
                     for name in _possible_event_stream_names:
-                        npz_filename = root / 'events' / d['folder_name'] / f'{name}.npy'
-                        if npz_filename.is_file():
-                            event_stream[f'{name}_npy'] = str(npz_filename)
+                        npy_filename = root / 'events' / d['folder_name'] / f'{name}.npy'
+                        if npy_filename.is_file():
+                            event_stream[f'{name}_npy'] = str(npy_filename)
 
-                    all_streams[block_index][seg_index]['events'][stream_name] = event_stream
+                    recording['streams']['events'][stream_name] = event_stream
 
-    # TODO for later: check stream / channel consistency across segment
+            folder_structure[node_name]['experiments'][experiment_id]['recordings'][recording_id] \
+                = recording
 
-    return all_streams, nb_block, nb_segment_per_block
+    # now create all_streams, nb_block, nb_segment_per_block
+    # nested dictionary: block_index > seg_index > data_type > stream_name
+    all_streams = {}
+    nb_segment_per_block = {}
+    recording_node = folder_structure[list(folder_structure.keys())[0]]
+
+    # nb_block needs to be consistent across record nodes. Use the first one
+    nb_block = len(recording_node['experiments'])
+
+    for node_id, recording_node in folder_structure.items():
+        exp_ids_sorted = sorted(list(recording_node['experiments'].keys()))
+        for block_index, exp_id in enumerate(exp_ids_sorted):
+            experiment = recording_node['experiments'][exp_id]
+            nb_segment_per_block[block_index] = len(experiment['recordings'])
+            if block_index not in all_streams:
+                all_streams[block_index] = {}
+
+            rec_ids_sorted = sorted(list(experiment['recordings'].keys()))
+            for seg_index, rec_id in enumerate(rec_ids_sorted):
+                recording = experiment['recordings'][rec_id]
+                if seg_index not in all_streams[block_index]:
+                    all_streams[block_index][seg_index] = {}
+                for stream_type in recording['streams']:
+                    if stream_type not in all_streams[block_index][seg_index]:
+                        all_streams[block_index][seg_index][stream_type] = {}
+                    for stream_name, signal_stream in recording['streams'][stream_type].items():
+                        all_streams[block_index][seg_index][stream_type][stream_name] = \
+                            signal_stream
+
+    # natural sort possible experiment names
+    experiment_order = np.argsort([int(exp.replace('experiment', ''))
+                                   for exp in possible_experiment_names])
+    possible_experiment_names = list(np.array(possible_experiment_names)[experiment_order])
+
+    return folder_structure, all_streams, nb_block, nb_segment_per_block, possible_experiment_names
+
+
+def check_folder_consistency(folder_structure, possible_experiment_names=None):
+    # check that experiment names are the same for differend record nodes
+    if len(folder_structure) > 1:
+        experiments = None
+        for node in folder_structure.values():
+            experiments_node = node['experiments']
+            if experiments is None:
+                experiments = experiments_node
+            experiment_names = [e['name'] for e_id, e in experiments.items()]
+            assert all(ename['name'] in experiment_names for ename in experiments_node.values()), \
+                ("Inconsistent experiments across recording nodes!")
+
+    # check that "continuous" streams are the same across multiple segments (recordings)
+    experiments = folder_structure[list(folder_structure.keys())[0]]['experiments']
+    for exp_id, experiment in experiments.items():
+        segment_stream_names = None
+        if len(experiment['recordings']) > 1:
+            for rec_id, recording in experiment['recordings'].items():
+                stream_names = sorted(list(recording['streams']['continuous'].keys()))
+                if segment_stream_names is None:
+                    segment_stream_names = stream_names
+                assert segment_stream_names == stream_names, \
+                    ("Inconsistent continuous streams across segments! Streams for different "
+                     "segments in the same experiment must be the same. Check your open ephys "
+                     "folder.")
+
+    # check that "continuous" streams across blocks (experiments)
+    block_stream_names = None
+    if len(experiments) > 1:
+        for exp_id, experiment in experiments.items():
+            # use 1st segment
+            rec_ids = list(experiment['recordings'])
+            stream_names = list(experiment['recordings'][rec_ids[0]]['streams']['continuous'].keys())
+            stream_names = sorted(stream_names)
+            if block_stream_names is None:
+                block_stream_names = stream_names
+            assert block_stream_names == stream_names, \
+                (f"Inconsistent continuous streams across blocks (experiments)! Streams for "
+                 f"different experiments in the same folder must be the same. You can load a "
+                 f"subset of experiments with the 'experiment_names' argument: "
+                 f"{possible_experiment_names}")
